# ----------------------------------------------------------------------
# This file was generated by /Users/humberto.a.sanchez.ii/PycharmProjects/PyUt/venv-pyenv-3.9.0/bin/img2py
#
from wx.lib.embeddedimage import PyEmbeddedImage

embeddedImage: PyEmbeddedImage = PyEmbeddedImage(
    b'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAK3RFWHRDcmVhdGlvbiBUaW1l'
    b'AEZyIDE1IE5vdiAyMDAyIDEwOjA4OjI5ICswMTAw9aJixgAAAAd0SU1FB9MDBgszJYmPv7sA'
    b'AAAJcEhZcwAACxIAAAsSAdLdfvwAAAAEZ0FNQQAAsY8L/GEFAAACp0lEQVR42qWTy08TURTG'
    b'v3Y69A0UqIi8IijUIKgsFJMuiEBIDBAJ6sLEdKUhGP8CE7cGd8aFIWHhhhAXJqQLIPGtKMZE'
    b'E0EeFtuS8mg7rfQx7byn4zDGSRBdcRZ3cb6c3/3uuecABwzD34lrS43uPIURp8XVZzbaj/Gy'
    b'pMTon4FIJubn6pVHkXOZ1H8Bw4lTQ/ZE5XhHbVepy+pElAshxSYg8Dw2d5J4HV5OhpyUL3tB'
    b'mt4HuEG1XqnnTk721A4S4cICZn7MYnmJgNPsRMmhLTQ63ChIEqZDa+KqIXEJA9Agxt3j6pKn'
    b'yhKvGO+s7ScWhLegxBBWYhv40P8RLy4+w3o6DkVRwIkiOmvqSHuSfIxJuHTAzjZ9+3x1V3GY'
    b'XUSKj0GRdhUDaCkDuaDAREADyLKsQdpdVW5EMKwD7ObSAbetHJ+p9wjEN5BjsiAJA0gjqYFI'
    b'kgAt5BCV0kjKOdgtJkBGvw6wmhzH36zPY3P1CLzyLbz8msIWTYEgVFl1kFKduGxnMdrqh89z'
    b'F5yxALOBaNIBjCDCVqRgjf8GX/N1PO+ewwlbB1gpB7ZAo+fwZYw2jcFjb8G7zBTyWQ6K/Lv5'
    b'GmA7SwXsquXTdQraZ5o1Ydo7h5IiFxymYjzwTGi5kWAvFkKfkKcliIwc0AHfkzF/JEXBabSh'
    b'zU3C+6pFKyAMhD4jI+FerK4vgd5RkGV4KCL8OoB3SQ+fBr9kCgXApJCotojonT+jF98MdmMl'
    b'uAiOBbIZHjEqn4CEsb2TOIWhBqHsibeuhmAkEYyYR0n5UfWHzAhEFpFLK8ikOUSjjMgQwgDu'
    b'YHb/Lkxg0JElx9uqKstsVhK82tw8K4BlZe3mOMVSklX2/Sn+5zLhPiqQV4dEQZ+1iGiQJQME'
    b'RlqDoL7Zqtq+h/RBN3hP/AKsNjhWr1LFPgAAAABJRU5ErkJggg==')
