# ----------------------------------------------------------------------
# This file was generated by /Users/humberto.a.sanchez.ii/PycharmProjects/PyUt/venv-pyenv-3.8.5-alternate/bin/img2py
#
from wx.lib.embeddedimage import PyEmbeddedImage

embeddedImage: PyEmbeddedImage = PyEmbeddedImage(
    b'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAALGPC/xhBQAAAHhl'
    b'WElmTU0AKgAAAAgABQESAAMAAAABAAEAAAEaAAUAAAABAAAASgEbAAUAAAABAAAAUgEoAAMA'
    b'AAABAAIAAIdpAAQAAAABAAAAWgAAAAAAAABHAAAAAQAAAEcAAAABAAKgAgAEAAAAAQAAABCg'
    b'AwAEAAAAAQAAABAAAAAAGq0jRgAAAAlwSFlzAAAK6wAACusBgosNWgAAAVlpVFh0WE1MOmNv'
    b'bS5hZG9iZS54bXAAAAAAADx4OnhtcG1ldGEgeG1sbnM6eD0iYWRvYmU6bnM6bWV0YS8iIHg6'
    b'eG1wdGs9IlhNUCBDb3JlIDYuMC4wIj4KICAgPHJkZjpSREYgeG1sbnM6cmRmPSJodHRwOi8v'
    b'd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgtbnMjIj4KICAgICAgPHJkZjpEZXNj'
    b'cmlwdGlvbiByZGY6YWJvdXQ9IiIKICAgICAgICAgICAgeG1sbnM6dGlmZj0iaHR0cDovL25z'
    b'LmFkb2JlLmNvbS90aWZmLzEuMC8iPgogICAgICAgICA8dGlmZjpPcmllbnRhdGlvbj4xPC90'
    b'aWZmOk9yaWVudGF0aW9uPgogICAgICA8L3JkZjpEZXNjcmlwdGlvbj4KICAgPC9yZGY6UkRG'
    b'Pgo8L3g6eG1wbWV0YT4KGV7hBwAAAtNJREFUOBFNUz1MU1EU/t5vaXmFZyrGRLAYiaibITg6'
    b'OMiCP4mDi4PGuBlWFwd3d5cOxMGRzRijgwNISowmQsSAghYqFWmx9Nn2vXvfj9+tBb3J9849'
    b'L/f7zrnnnKvhv9UaweCOwJSVcyatQ+5IJGM0artrpZr/4k0ThYe/scbjgkj2adr+JhjFTX9o'
    b'8HHvtRtZI5cDNteBagXw6pDb21hf3PSK1eDBrTqeklMnIsU11EeRw/HzT5w793r03W1g5R2w'
    b'853HakCjDqPdwuE+pFwpJ3J+0ngt8Zm0NpEYNaYdHx987tyd6sHSPKNuAnHMRENAMtt9CAHX'
    b'jjS09fML7XipmmCLAr6+52PKuXw9i2/LjMjoihwoYvAXwqevQD+UyOWc7CULl0keJmzTdjOT'
    b'+sAAStPT2Os7BteMkDET9Fg27PQA4BwFdN40lYK9yqttSowa1hjVz1CgZOpu3wgqG0jtVZGa'
    b'WWIUgSSTARVg8oROHKz7V9Fcn4dr2ScokOf/Xj2U7MjOFvqzNoSMEEmJmIhEBMkTB/1S+yTB'
    b'duwgiDX12yVss16rrw159dNpPcTwo9uw0hno6R5odgqGYUHrnDWQ2DZkmGBDy6Hll7+R3Flm'
    b'eTd4caby87QZR8guvGQYtleyA7wKRIKEpq1qynTmzGPQon4URciKd2ZB6LMChY8fSh4Eq8xq'
    b'J6oDIQWoQz5+swiCNVy1+7Fs5aH9+Oq9/EsuUaRpzAk0zsmo4cho4jD7HHJ8QzIDRgyo49O+'
    b'S4ZQ1E/CrazEhaY3swGUSZ4lytRG/EzgixNgL2nq48JKp35paVTiLFYwgKJ2EjVfR+r7J6/Q'
    b'8mbec9S65EXa5v5bUEJHThm4cBG4MmqaY/2WnRecqaoQpbdhuPyKabcAPg4UibfEDsHR/LeU'
    b'iGpNnjjbtcpXSz0edWdVPGWV33lM/wvwH5RvE1mit7un6TzhJq3X3R+Mxx9lmlGfclIKOAAA'
    b'AABJRU5ErkJggg==')

